import click

from sagasu import util as u
from sagasu.config import ConfigUtil
from sagasu.crawler import CrawlerEngine
from sagasu.engine import SearchEngine


@click.command()
@click.argument("mode")
def app(mode):
    if mode not in ["search", "crawl", "indexing"]:
        print("choice in [search, crawl, indexing]")
        return

    if not u.is_exist("/config/config.yml"):
        print("please set ~/.sagasu/config/config.yml")
        return

    config = ConfigUtil().load()
    engine = SearchEngine(config)

    if mode == "indexing":
        crawler_engine = CrawlerEngine(config.sources)
        crawler_engine.crawl_all()
        engine.indexing()
    elif mode == "search":
        word = input("let's type search word >>> ")
        resources = engine.word_search(word)
        resources = set(resources)
        for resource in resources:
            print(
                f"""
  [URI]
    {resource.uri}
  [Sentence]
    {resource.sentence[:30]}...
      """
            )


if __name__ == "__main__":
    app()



import pickle
from functools import reduce
from pathlib import Path
from typing import List

from sagasu.config import ConfigModel, SourceModel
from sagasu.indexer import WordNgramIndexer
from sagasu.entity import Resource, IndexedResource
from sagasu.repository import Repository, TwitterRepository, ScrapboxRepository
from sagasu.util import SAGASU_WORKDIR


class SearchEngine:
    def __init__(self, config: ConfigModel):
        self.indexers = [
            WordNgramIndexer(n=1),
            WordNgramIndexer(n=2),
            WordNgramIndexer(n=3),
        ]
        self.indexed_resource = (
            self.load_indexed()
            if Path(f"{SAGASU_WORKDIR}/indexed").exists()
            else IndexedResource()
        )
        self.config = config
        self.repositories: List[Repository] = [
            self.load_repository(source) for source in self.config.sources
        ]

    def load_indexed(self):
        p = reduce(
            lambda pre, cur: max(pre, cur, key=lambda f: f.stat().st_ctime),
            [p for p in Path(f"{SAGASU_WORKDIR}/indexed").iterdir()],
        )
        p.absolute()
        with p.open("rb") as f:
            indexed = pickle.load(f)
        return indexed

    @staticmethod
    def load_repository(source: SourceModel) -> Repository:
        if source.source_type == "twitter":
            return TwitterRepository()
        elif source.source_type == "scrapbox":
            return ScrapboxRepository()

    def _load_all(self) -> List[Resource]:
        resources = []
        for repository in self.repositories:
            _resources = repository.load()
            resources += _resources
        return resources

    def indexing(self):
        resources = self._load_all()
        for resource in resources:
            for indexer in self.indexers:
                self.indexed_resource = indexer(resource, self.indexed_resource)
        return

    def word_search(self, word: str) -> List[Resource]:
        if {} == self.indexed_resource:
            raise Exception("run indexing before call searching")
        elif word not in self.indexed_resource:
            return []
        return self.indexed_resource[word]



from dataclasses import dataclass
from typing import List

import yaml

from sagasu import util


@dataclass
class SourceModel:
    source_type: str
    target: str


@dataclass
class ConfigModel:
    sources: List[SourceModel]


class ConfigUtil:
    def __init__(self):
        self.path = f"{util.SAGASU_WORKDIR}/config/config.yml"

    def load(self) -> ConfigModel:
        with open(self.path) as f:
            _config = yaml.safe_load(f)
            config = ConfigModel(
                sources=[
                    SourceModel(source_type=c["source_type"], target=c["target"])
                    for c in _config["sources"]
                ]
            )
        return config



from dataclasses import dataclass
from typing import List, TypedDict, Union


@dataclass
class Resource:
    uri: str
    sentence: str

    def __eq__(self, other):
        if type(other) is not Resource:
            return False
        elif other.uri == self.uri:
            return True
        else:
            return False

    def __hash__(self):
        return hash(self.uri)


@dataclass
class TokenizedResource:
    index: List[str]
    resource: Resource


@dataclass
class IndexedResource(TypedDict):
    index: str
    resource: List[Resource]


@dataclass
class SampleResource(Resource):
    pass


@dataclass
class TwitterResource(Resource):
    image_urls: Union[List[str], None]
    image_captions: Union[List[str], None]


@dataclass
class ScrapboxResource(Resource):
    image_urls: Union[List[str], None]
    image_captions: Union[List[str], None]


@dataclass
class SlackResource(Resource):
    pass



import tensorflow as tf
import requests


class BahdanauAttention(tf.keras.Model):
    def __init__(self, units):
        super(BahdanauAttention, self).__init__()
        self.W1 = tf.keras.layers.Dense(units)
        self.W2 = tf.keras.layers.Dense(units)
        self.V = tf.keras.layers.Dense(1)

    def call(self, features, hidden):
        hidden_with_time_axis = tf.expand_dims(hidden, 1)
        score = tf.nn.tanh(self.W1(features) + self.W2(hidden_with_time_axis))
        attention_weights = tf.nn.softmax(self.V(score), axis=1)
        context_vector = attention_weights * features
        context_vector = tf.reduce_sum(context_vector, axis=1)

        return context_vector, attention_weights


class CNNEncoder(tf.keras.Model):
    def __init__(self, embedding_dim):
        super(CNNEncoder, self).__init__()
        self.fc = tf.keras.layers.Dense(embedding_dim)

    def call(self, x):
        x = self.fc(x)
        x = tf.nn.relu(x)
        return x


class RNNDecoder(tf.keras.Model):
    def __init__(self, embedding_dim, units, vocab_size):
        super(RNNDecoder, self).__init__()
        self.units = units

        self.embedding = tf.keras.layers.Embedding(vocab_size, embedding_dim)
        self.gru = tf.keras.layers.GRU(
            self.units,
            return_sequences=True,
            return_state=True,
            recurrent_initializer="glorot_uniform",
        )
        self.fc1 = tf.keras.layers.Dense(self.units)
        self.fc2 = tf.keras.layers.Dense(vocab_size)

        self.attention = BahdanauAttention(self.units)

    def call(self, x, features, hidden):
        context_vector, attention_weights = self.attention(features, hidden)
        x = self.embedding(x)
        x = tf.concat([tf.expand_dims(context_vector, 1), x], axis=-1)
        output, state = self.gru(x)
        x = self.fc1(output)
        x = tf.reshape(x, (-1, x.shape[2]))
        x = self.fc2(x)
        return x, state, attention_weights

    def reset_state(self, batch_size):
        return tf.zeros((batch_size, self.units))


def pre_processing(media_url: str):
    image = tf.image.decode_image(
        requests.get(media_url).content, channels=3, dtype=tf.float32
    )
    l = m if (m := max(image.shape)) > 224 else 224
    image = tf.image.resize_with_crop_or_pad(image, target_height=l, target_width=l)
    image = tf.image.resize(image, size=(224, 224))
    image = tf.reshape(image, [1, 224, 224, 3])
    return image


def image_captioning_from_url(media_url: str) -> str:
    image = pre_processing(media_url)
    return image_captioning(image)


def image_captioning(image) -> str:
    image_model = tf.keras.applications.MobileNetV2(
        include_top=False, weights="imagenet", input_shape=[224, 224, 3]
    )
    new_input = image_model.input
    hidden_layer = image_model.layers[-1].output

    image_features_extract_model = tf.keras.Model(new_input, hidden_layer)
    features = image_features_extract_model(image)
    features = tf.reshape(features, (features.shape[0], -1, features.shape[3]))

    enc = CNNEncoder(embedding_dim=256)
    dec = RNNDecoder(embedding_dim=256, units=512, vocab_size=9880)

    dec.load_weights("model/dec/dec_save_weights")

    enc.load_weights("model/enc/enc_save_weights")
    encoded = enc(features)
    hidden = dec.reset_state(batch_size=1)

    with open("model/tokenize/token.json") as f:
        s = f.readline()
    tokenizer = tf.keras.preprocessing.text.tokenizer_from_json(s)
    result = []
    dec_input = tf.expand_dims([tokenizer.word_index["<start>"]], 0)

    for i in range(100):
        predictions, hidden, attention_weights = dec(dec_input, encoded, hidden)

        predicted_id = tf.random.categorical(predictions, 1)[0][0].numpy()
        result.append(tokenizer.index_word[predicted_id])

        if tokenizer.index_word[predicted_id] == "<end>":
            break
        else:
            dec_input = tf.expand_dims([predicted_id], 0)
    return "".join(result)[:-5]


if __name__ == "__main__":
    _images = tf.random.normal([1, 224, 224, 3])

    res = image_captioning(_images)
    print(res)



import os
from pathlib import Path
from typing import List

import pandas as pd

from sagasu import entity as m
from sagasu import util as u
from sagasu import crawler as c


class Repository:
    def load(self) -> List[m.Resource]:
        raise NotImplementedError("")


class SampleRepository(Repository):
    def __init__(self):
        u.mkdir_p((p := f"{u.SAGASU_WORKDIR}/sample/sample.txt"))
        with open(p, "w") as f:
            f.write(
                """
吾輩は猫である。
親譲りの無鉄砲で小供の時から損ばかりしている。
"""
            )

    def load(self) -> List[m.SampleResource]:
        with open(p := f"{u.SAGASU_WORKDIR}/sample/sample.txt") as f:
            document = f.readlines()
            document = "".join(document)
        return [m.SampleResource(uri=p, sentence=document)]


class TwitterRepository(Repository):
    def load(self) -> List[m.TwitterResource]:
        dir_path = Path(f"{c.CRAWLER_WORK_DIR}/twitter/uri-sentence")
        uri_sentence_dfs = []
        for file in dir_path.iterdir():
            if not file.is_file():
                continue
            df = pd.read_csv(file, sep="\t")
            uri_sentence_dfs.append(df)
        uri_sentence_df = pd.concat(uri_sentence_dfs).reset_index()

        dir_path = Path(f"{c.CRAWLER_WORK_DIR}/twitter/uri-media")
        uri_media_dfs = []
        for file in dir_path.iterdir():
            if not file.is_file():
                continue
            df = pd.read_csv(file, sep="\t")
            uri_media_dfs.append(df)
        uri_media_df = pd.concat(uri_media_dfs).reset_index()

        df = pd.merge(uri_sentence_df, uri_media_df)
        # media_url1  media_url2  media_url3  media_url4  media_caption1
        resources = [
            m.TwitterResource(
                uri=r[1].uri,
                sentence=r[1].sentence,
                image_urls=[
                    r[1].media_url1,
                    r[1].media_url2,
                    r[1].media_url3,
                    r[1].media_url4,
                ],
                image_captions=[
                    r[1].media_caption1,
                    r[1].media_caption2,
                    r[1].media_caption3,
                    r[1].media_caption4,
                ],
            )
            for r in df.iterrows()
        ]

        return resources


class ScrapboxRepository(Repository):
    def load(self) -> List[m.ScrapboxResource]:
        dir_path = Path(f"{c.CRAWLER_WORK_DIR}/scrapbox/uri-sentence")
        uri_sentence_dfs = []
        for file in dir_path.iterdir():
            if not file.is_file():
                continue
            df = pd.read_csv(file, sep="\t")
            uri_sentence_dfs.append(df)
        uri_sentence_df = pd.concat(uri_sentence_dfs).reset_index()

        dir_path = Path(f"{c.CRAWLER_WORK_DIR}/scrapbox/uri-media")
        uri_media_dfs = []
        for file in dir_path.iterdir():
            if not file.is_file():
                continue
            df = pd.read_csv(file, sep="\t")
            uri_media_dfs.append(df)
        uri_media_df = pd.concat(uri_media_dfs).reset_index()

        df = pd.merge(uri_sentence_df, uri_media_df)
        # media_url1  media_url2  media_url3  media_url4  media_caption1
        resources = [
            m.ScrapboxResource(
                uri=r[1].uri,
                sentence=r[1].sentence,
                image_urls=[
                    r[1].media_url1,
                    r[1].media_url2,
                    r[1].media_url3,
                    r[1].media_url4,
                ],
                image_captions=[
                    r[1].media_caption1,
                    r[1].media_caption2,
                    r[1].media_caption3,
                    r[1].media_caption4,
                ],
            )
            for r in df.iterrows()
        ]

        return resources



from typing import List
import os
import pickle

import spacy
import datetime

from sagasu.entity import Resource, IndexedResource
from sagasu.util import mkdir_p

nlp = spacy.load("ja_ginza")
JST = datetime.timezone(datetime.timedelta(hours=+9), "JST")


def simple_n_gram(sentence: str, n: int) -> List[str]:
    index = [sentence[idx : idx + n] for idx in range(len(sentence) - n + 1)]
    return index


def word_n_gram(sentence: str, n: int = 1) -> List[str]:
    doc = nlp(sentence)
    tokens = []
    for sent in doc.sents:
        for token in sent:
            tokens.append(token.orth_)
    index = ["".join(tokens[idx : idx + n]) for idx in range(len(tokens) - n + 1)]
    return index


def dump(func):
    def runner(*args, **kwargs):
        result = func(*args, **kwargs)
        t = datetime.datetime.now(JST)
        path_prefix = f"{os.getenv('HOME')}/.sagasu"
        filename_prefix = (
            f"/{t.year}-"
            f"{m if len(m := str(t.month)) == 2 else f'0{m}'}-"
            f"{d if len(d := str(t.day)) == 2 else f'0{d}'}-"
            f"{h if len(h := str(t.hour)) == 2 else f'{h}'}"
        )

        mkdir_p(file_path := path_prefix + "/indexed" + filename_prefix + ".pkl")
        with open(file_path, "wb") as f:
            pickle.dump(result, f)
        return result

    return runner


class Indexer:
    def __call__(self, resource: Resource, indexed: IndexedResource) -> IndexedResource:
        raise NotImplementedError("not implemented")


class WordUnigramIndexer(Indexer):
    def __call__(self, resource: Resource, indexed: IndexedResource) -> IndexedResource:
        indices = word_n_gram(resource.sentence)
        for index in indices:
            if index not in indexed:
                indexed[index] = [resource]
            else:
                indexed[index].append(resource)
        return indexed


class WordNgramIndexer(Indexer):
    def __init__(self, n):
        self.n = n

    @dump
    def __call__(self, resource: Resource, indexed: IndexedResource) -> IndexedResource:
        indices = word_n_gram(resource.sentence, self.n)
        for index in indices:
            if index not in indexed:
                indexed[index] = [resource]
            else:
                indexed[index].append(resource)
        return indexed


if __name__ == "__main__":
    print(simple_n_gram("毎日の料理を楽しみにする", 2))
    print(word_n_gram("毎日の料理を楽しみにする"))
    print(word_n_gram("毎日の料理を楽しみにする", 2))



from pathlib import Path
import os


SAGASU_WORKDIR = f"{os.getenv('HOME')}/.sagasu"


def mkdir_p(file_path):
    (p := Path(file_path).parent).mkdir(parents=True, exist_ok=True)
    p.touch()


def is_exist(file_path):
    return Path(f"{SAGASU_WORKDIR}{file_path}").exists()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['sagasu']

package_data = \
{'': ['*']}

install_requires = \
['click>=7.1.2,<8.0.0',
 'ginza>=3.1.2,<4.0.0',
 'numpy>=1.18.4,<2.0.0',
 'pandas>=1.0.3,<2.0.0',
 'pillow>=7.1.2,<8.0.0',
 'pyyaml>=5.3.1,<6.0.0',
 'spacy>=2.2.4,<3.0.0',
 'tqdm>=4.46.0,<5.0.0',
 'tweepy>=3.8.0,<4.0.0']

entry_points = \
{'console_scripts': ['sagasu = sagasu.app:app']}

setup_kwargs = {
    'name': 'sagasu',
    'version': '0.4.8',
    'description': 'sagasu is search all my contents',
    'long_description': None,
    'author': 'funwarioisii',
    'author_email': None,
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



import datetime
import json
import os
import re
from pathlib import Path
from time import sleep
from typing import List

import pandas as pd
import requests as req
import tweepy
from tqdm import tqdm

from sagasu import entity as m
from sagasu import util
from sagasu.config import SourceModel

if os.getenv('SAGASU_CAPTION'):
    from sagasu.image_captioning import image_captioning_from_url

CONSUMER_KEY = os.getenv("CONSUMER_KEY")
CONSUMER_SECRET = os.getenv("CONSUMER_SECRET")
ACCESS_TOKEN = os.getenv("ACCESS_TOKEN")
ACCESS_TOKEN_SECRET = os.getenv("ACCESS_TOKEN_SECRET")

CRAWLER_WORK_DIR = util.SAGASU_WORKDIR + "/crawler"

JST = datetime.timezone(datetime.timedelta(hours=+9), "JST")


class Crawler:
    def __init__(self, source: SourceModel):
        self.path_prefix = CRAWLER_WORK_DIR
        self.resources: List[m.Resource] = []
        self.source = source

    def _collect(self) -> List[m.Resource]:
        raise NotImplementedError("not implemented")

    def _dump(self):
        raise NotImplementedError("not implemented")

    def __call__(self, *args, **kwargs):
        self.resources = self._collect()
        self._dump()

    @staticmethod
    def _image_captioning(media_url: str) -> str:
        return image_captioning_from_url(media_url)


class CrawlerEngine:
    def __init__(self, sources: List[SourceModel]):
        self.resources = []
        self.crawlers: List[Crawler] = [self.load_crawler(source) for source in sources]

    def load_crawler(self, source: SourceModel):
        if source.source_type == "twitter":
            return TwitterFavoriteCrawler(source)
        elif source.source_type == "scrapbox":
            return ScrapboxCrawler(source)

    def crawl_all(self):
        for crawler in self.crawlers:
            crawler()
            self.resources += crawler.resources


class TwitterFavoriteCrawler(Crawler):
    def __init__(self, source: SourceModel):
        super().__init__(source)
        self.path_prefix += "/twitter"
        self.resources: List[m.TwitterResource] = []

    def _collect(self) -> List[m.Resource]:
        statuses: List[tweepy.models.Status] = self._load_favorites()

        resources = []
        for status in statuses:
            uri = f"https://twitter.com/_/status/{status.id}"
            sentence = "".join(status.text.split("\n"))
            media_urls = []
            media_captions = []

            if "media" in status.entities:
                for media in status.extended_entities["media"]:
                    media_url = media["media_url"]
                    media_caption = (
                        self._image_captioning(media_url)
                        if os.getenv("SAGASU_IMAGE_CAPTIONING")
                        else "empty"
                    )
                    media_urls.append(media_url)
                    media_captions.append(media_caption)

            resources.append(
                m.TwitterResource(
                    uri=uri,
                    sentence=sentence,
                    image_urls=media_urls,
                    image_captions=media_captions,
                )
            )

        return resources

    def _load_favorites(self) -> List[tweepy.models.Status]:
        auth = tweepy.OAuthHandler(CONSUMER_KEY, CONSUMER_SECRET)
        auth.set_access_token(ACCESS_TOKEN, ACCESS_TOKEN_SECRET)
        api = tweepy.API(auth)

        statuses = []
        progress_bar = tqdm(total=10)
        progress_bar.set_description("collecting favorites")
        for n in range(10):
            statuses += api.favorites(self.source.target, page=n + 1)
            progress_bar.update(1)

        return statuses

    def _dump(self):
        t = datetime.datetime.now(JST)
        filename_prefix = (
            f"/{t.year}-"
            f"{m if len(m := str(t.month)) == 2 else f'0{m}'}-"
            f"{d if len(d := str(t.day)) == 2 else f'0{d}'}-"
            f"{h if len(h := str(t.hour)) == 2 else f'{h}'}"
        )
        resource_type_prefix = "/uri-sentence"

        filename = self.path_prefix + resource_type_prefix + filename_prefix + ".tsv"
        util.mkdir_p(filename)

        uri_sentence_pairs = list(
            map(lambda resource: [resource.uri, resource.sentence], self.resources)
        )
        pd.DataFrame(uri_sentence_pairs, columns=["uri", "sentence"]).to_csv(
            filename, sep="\t"
        )

        resource_type_prefix = "/uri-media"
        filename = self.path_prefix + resource_type_prefix + filename_prefix + ".tsv"
        util.mkdir_p(filename)

        def extract(resource: List[str]):
            _1, _2, _3, _4 = (resource + ["empty", "empty", "empty", "empty"])[:4]
            return _1, _2, _3, _4

        uri_media_pairs = list(
            map(
                lambda resource: [resource.uri]
                + list(extract(resource.image_urls))
                + list(extract(resource.image_captions)),
                self.resources,
            )
        )
        pd.DataFrame(
            uri_media_pairs,
            columns=[
                "uri",
                "media_url1",
                "media_url2",
                "media_url3",
                "media_url4",
                "media_caption1",
                "media_caption2",
                "media_caption3",
                "media_caption4",
            ],
        ).to_csv(filename, sep="\t")
        return


class ScrapboxCrawler(Crawler):
    def __init__(self, source: SourceModel):
        super().__init__(source=source)
        self.path_prefix += "/scrapbox"
        self.target = self.source.target

    def _collect(self) -> List[m.ScrapboxResource]:
        res = req.get(page_url := f"https://scrapbox.io/api/pages/{self.target}")
        resources = []
        pages = json.loads(res.text)["pages"]
        progress_bar = tqdm(total=len(pages))
        progress_bar.set_description("collecting scrapbox")
        for page in pages:
            title = t if not "/" in (t := page["title"]) else t.replace("/", "%2F")
            page = json.loads(req.get(uri := f"{page_url}/{title}").text)
            sentence = " ".join(lines := list(map(lambda p: p["text"], page["lines"])))
            image_uris = [s[1:-1] for s in lines if re.match(r"\[https://gyazo.com", s)]
            image_captions = ["empty" for _ in image_uris]
            resources.append(
                m.ScrapboxResource(
                    uri=uri,
                    sentence=sentence,
                    image_urls=image_uris,
                    image_captions=image_captions,
                )
            )
            sleep(0.5)
            progress_bar.update(1)
        return resources

    def _dump(self):
        t = datetime.datetime.now(JST)
        filename_prefix = (
            f"/{t.year}-"
            f"{mo if len(mo := str(t.month)) == 2 else f'0{mo}'}-"
            f"{d if len(d := str(t.day)) == 2 else f'0{d}'}-"
            f"{h if len(h := str(t.hour)) == 2 else f'{h}'}"
        )
        resource_type_prefix = "/uri-sentence"

        filename = self.path_prefix + resource_type_prefix + filename_prefix + ".tsv"
        util.mkdir_p(filename)
        uri_sentence_pairs = list(
            map(lambda resource: [resource.uri, resource.sentence], self.resources)
        )
        pd.DataFrame(uri_sentence_pairs, columns=["uri", "sentence"]).to_csv(
            filename, sep="\t"
        )

        resource_type_prefix = "/uri-media"
        filename = self.path_prefix + resource_type_prefix + filename_prefix + ".tsv"
        util.mkdir_p(filename)

        def extract(resource: List[str]):
            _1, _2, _3, _4 = (resource + ["empty", "empty", "empty", "empty"])[:4]
            return _1, _2, _3, _4

        uri_media_pairs = list(
            map(
                lambda resource: [resource.uri]
                + list(extract(resource.image_urls))
                + list(extract(resource.image_captions)),
                self.resources,
            )
        )
        pd.DataFrame(
            uri_media_pairs,
            columns=[
                "uri",
                "media_url1",
                "media_url2",
                "media_url3",
                "media_url4",
                "media_caption1",
                "media_caption2",
                "media_caption3",
                "media_caption4",
            ],
        ).to_csv(filename, sep="\t")
        return


class SlackCrawler(Crawler):
    def __init__(self, source: SourceModel):
        super().__init__(source)
        self.path_prefix += "/slack"

    def _collect(self) -> List[m.SlackResource]:
        for workspace in Path(self.path_prefix).iterdir():
            for file in workspace.iterdir():
                if not file.is_file():
                    continue
                with file.open() as f:
                    d = json.load(
                        f
                    )  # [{type: "text", attachments: [{title: "title", text: "text", }]}]
        pass

    def _dump(self):
        pass
